@@ -1,7 +1,7 @@
 /*
  * radius.h	Constants of the radius protocol.
  *
- * Version:	$Id: radius.h,v 1.43 2002/07/18 08:22:30 pam Exp $
+ * Version:	$Id: radius.h,v 1.46 2002/08/16 14:46:21 kkalev Exp $
  *
  */
 
@@ -28,6 +28,7 @@
 #define PW_STATUS_CLIENT		13
 
 #define	PW_USER_NAME			1
+#define	PW_USER_PASSWORD		2
 #define	PW_PASSWORD			2
 #define	PW_CHAP_PASSWORD		3
 #define	PW_NAS_IP_ADDRESS		4
@@ -157,6 +158,8 @@
 #define PW_MODULE_SUCCESS_MESSAGE	1075
 #define PW_MODULE_FAILURE_MESSAGE	1076
 #define PW_X99_FAST			1077
+#define PW_REWRITE_RULE			1078
+#define PW_SQL_GROUP			1079
 
 /*
  *	Integer Translations
