@@ -1,7 +1,7 @@
 /*
  * auth.c	User authentication.
  *
- * Version:	$Id: auth.c,v 1.108 2002/07/17 18:20:56 aland Exp $
+ * Version:	$Id: auth.c,v 1.112 2002/10/08 21:48:27 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
  * Copyright 2000  Jeff Carneal <jeff@apex.net>
  */
 
-static const char rcsid[] = "$Id: auth.c,v 1.108 2002/07/17 18:20:56 aland Exp $";
+static const char rcsid[] = "$Id: auth.c,v 1.112 2002/10/08 21:48:27 aland Exp $";
 
 #include "autoconf.h"
 #include "libradius.h"
@@ -473,6 +473,7 @@ int rad_authenticate(REQUEST *request)
 		 *	we added, which MUST NOT go back to the NAS.
 		 */
 		if (request->proxy_reply->vps) {
+			pairfree(&request->reply->vps);
 			request->reply->vps = request->proxy_reply->vps;
 			pairdelete(&request->reply->vps, PW_PROXY_STATE);
 			request->reply->code = request->proxy_reply->code;
@@ -823,8 +824,8 @@ autz_redo:
 
 			request->reply->code = PW_AUTHENTICATION_REJECT;
 			tmp = pairmake("Reply-Message", user_msg, T_OP_SET);
-			request->reply->vps = tmp;
-			
+
+			pairadd(&request->reply->vps, tmp);
 			rad_authlog("Login incorrect (external check failed)", 
 					request, 0);
 
@@ -861,13 +862,13 @@ autz_redo:
 	if (user_msg == NULL) {
 		reply_item = pairfind(request->reply->vps, PW_REPLY_MESSAGE);
 		while (reply_item) {
-		  radius_xlat(buf, sizeof(reply_item->strvalue),
-			      (char *)reply_item->strvalue, request, NULL);
-		strNcpy((char *)reply_item->strvalue, buf,
+			radius_xlat(buf, sizeof(reply_item->strvalue),
+				    (char *)reply_item->strvalue, request, NULL);
+			strNcpy((char *)reply_item->strvalue, buf,
 				sizeof(reply_item->strvalue));
-		reply_item->length = strlen((char *)reply_item->strvalue);
-		user_msg = NULL;
-		reply_item = pairfind(reply_item->next, PW_REPLY_MESSAGE);
+			reply_item->length = strlen((char *)reply_item->strvalue);
+			user_msg = NULL;
+			reply_item = pairfind(reply_item->next, PW_REPLY_MESSAGE);
 		}
 	}
 
@@ -897,16 +898,42 @@ autz_redo:
 
 	if (exec_program) 
 		free(exec_program);
-	return RLM_MODULE_OK;
+
+	/*
+	 *	Do post-authentication calls. ignoring the return code.
+	 *	If the post-authentication
+	 */
+	result = module_post_auth(request);
+	switch (result) {
+	  break;
+	  
+	  /*
+	   *	The module failed, or said to reject the user: Do so.
+	   */
+	case RLM_MODULE_FAIL:
+	case RLM_MODULE_REJECT:
+	case RLM_MODULE_USERLOCK:
+	case RLM_MODULE_INVALID:
+	  request->reply->code = PW_AUTHENTICATION_REJECT;
+	  result = RLM_MODULE_REJECT;
+	  break;
+
+	  /*
+	   *	The module had a number of OK return codes.
+	   */
+	case RLM_MODULE_NOTFOUND:
+	case RLM_MODULE_NOOP:
+	case RLM_MODULE_UPDATED:
+	case RLM_MODULE_OK:
+	case RLM_MODULE_HANDLED:
+	  result = RLM_MODULE_OK;
+	  break;
+	}
+
+	return result;
 }
 
 
-/*
- *	These definitions are local, and shouldn't be used by anyone else.
- */
-#define PW_ENCODED  0
-#define PW_DECODED  1
-
 /*
  * Find the password pair, decode pass if
  * needed, and return the value pair.  If
@@ -942,32 +969,6 @@ VALUE_PAIR *rad_getpass(REQUEST *request) {
 		request->password = auth_item;
 	}
 
-
-	/*
-	 *	If we proxied already, it's been decoded
-	 *	Or if the decoded flag is set...just return
-	 */
-	if ((request->proxy != NULL) ||
-			(auth_item->lvalue == PW_DECODED)) {
-		return auth_item;
-	}
-
-	/* 
-	 *	If we get here, we have to decode the password.
-	 */
-	rad_pwdecode((char *)auth_item->strvalue,
-			auth_item->length, request->secret,
-			(char *)request->packet->vector);
-
-	/* 
-	 *	Set lvalue to PW_DECODED so we know not to
-	 *	decode next time we get here
-	 */
-	auth_item->lvalue = PW_DECODED;
-
-	/* ignore more than one trailing '\0' */
-	auth_item->length = strlen((char *)auth_item->strvalue);
-
 	return auth_item;
 }
 
