@@ -1,7 +1,7 @@
 /*
  * files.c	Read config files into memory.
  *
- * Version:     $Id: files.c,v 1.66 2002/06/21 19:57:26 cparker Exp $
+ * Version:     $Id: files.c,v 1.71 2002/10/29 22:55:26 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
  * Copyright 2000  Alan DeKok <aland@ox.org>
  */
 
-static const char rcsid[] = "$Id: files.c,v 1.66 2002/06/21 19:57:26 cparker Exp $";
+static const char rcsid[] = "$Id: files.c,v 1.71 2002/10/29 22:55:26 aland Exp $";
 
 #include "autoconf.h"
 #include "libradius.h"
@@ -41,9 +41,7 @@ static const char rcsid[] = "$Id: files.c,v 1.66 2002/06/21 19:57:26 cparker Exp
 
 #include "radiusd.h"
 
-extern int proxy_dead_time;
-
-REALM *realms = NULL;
+int maximum_proxies;
 
 /*
  *	Free a PAIR_LIST
@@ -348,7 +346,7 @@ static void debug_pair_list(PAIR_LIST *pl)
 /*
  *	Free a REALM list.
  */
-static void realm_free(REALM *cl)
+void realm_free(REALM *cl)
 {
 	REALM *next;
 
@@ -373,9 +371,9 @@ int read_realms_file(const char *file)
 	int lineno = 0;
 	REALM *c, **tail;
 
-	realm_free(realms);
-	realms = NULL;
-	tail = &realms;
+	realm_free(mainconfig.realms);
+	mainconfig.realms = NULL;
+	tail = &mainconfig.realms;
 
 	if ((fp = fopen(file, "r")) == NULL) {
 		/* The realms file is not mandatory.  If it exists it will
@@ -407,8 +405,8 @@ int read_realms_file(const char *file)
 			c->auth_port = atoi(s);
 			c->acct_port = c->auth_port + 1;
 		} else {
-			c->auth_port = auth_port;
-			c->acct_port = acct_port;
+			c->auth_port = PW_AUTH_UDP_PORT;
+			c->acct_port = PW_ACCT_UDP_PORT;
 		}
 
 		if (strcmp(hostnm, "LOCAL") == 0) {
@@ -502,15 +500,15 @@ void realm_disable(uint32_t ipaddr, int port)
 	time_t now;
 
 	now = time(NULL);
-	for(cl = realms; cl; cl = cl->next)
+	for(cl = mainconfig.realms; cl; cl = cl->next)
 		if ((ipaddr == cl->ipaddr) && (port == cl->auth_port)) {
 			cl->active = FALSE;
-			cl->wakeup = now + proxy_dead_time;
+			cl->wakeup = now + mainconfig.proxy_dead_time;
 			radlog(L_PROXY, "marking authentication server %s:%d for realm %s dead",
 				cl->server, port, cl->realm);
 		} else if ((ipaddr == cl->acct_ipaddr) && (port == cl->acct_port)) {
 			cl->acct_active = FALSE;
-			cl->acct_wakeup = now + proxy_dead_time;
+			cl->acct_wakeup = now + mainconfig.proxy_dead_time;
 			radlog(L_PROXY, "marking accounting server %s:%d for realm %s dead",
 				cl->server, port, cl->realm);
 		}
@@ -521,13 +519,14 @@ void realm_disable(uint32_t ipaddr, int port)
  */
 REALM *realm_find(const char *realm, int acct)
 {
-	REALM *cl;
+	REALM *cl, *realmptr;
 	REALM *default_realm = NULL;
+	REALM *rr_array[maximum_proxies];
 	time_t now;
-	int dead_match = 0;
+	int dead_match = 0, efform = 0, p = 0, i = 0;
+	static int proxy_return;
 
 	now = time(NULL);
-
 	/*
 	 *	If we're passed a NULL realm pointer,
 	 *	then look for a "NULL" realm string.
@@ -535,8 +534,8 @@ REALM *realm_find(const char *realm, int acct)
 	if (realm == NULL) {
 		realm = "NULL";
 	}
-
-	for (cl = realms; cl; cl = cl->next) {
+	
+	for (cl = mainconfig.realms; cl; cl = cl->next) {
 		/*
 		 *	Wake up any sleeping realm.
 		 */
@@ -560,8 +559,8 @@ REALM *realm_find(const char *realm, int acct)
 			 *	exact matches for this realm which are
 			 *	dead.
 			 */
-			if ((!proxy_fallback) &&
-			    (strcmp(cl->realm, realm) == 0)) {
+			if ((!mainconfig.proxy_fallback) &&
+			    (strcasecmp(cl->realm, realm) == 0)) {
 				dead_match = 1;
 			}
 			continue;
@@ -570,8 +569,107 @@ REALM *realm_find(const char *realm, int acct)
 		/*
 		 *	If it matches exactly, return it.
 		 */
-		if (strcmp(cl->realm, realm) == 0) {
-			return cl;
+		if (strcasecmp(cl->realm, realm) == 0) {
+                        /*
+                         *    Seperate parsing of list for round-robin
+                         */
+			if(cl->total > 1 && cl->ldflag == 1) {
+				/*Get all of the realms from initial list*/
+				for(i = 1; i <= cl->total; i++) {
+					for(realmptr = mainconfig.realms; realmptr; 
+                                            realmptr = realmptr->next) {
+						if((strcasecmp(realmptr->realm,
+                                                   cl->realm)) == 0 &&
+						   (realmptr->node == i)) {
+							rr_array[i] = realmptr;
+							if(realmptr->active == TRUE)
+								p++;
+							break;
+						}
+						else {
+							continue;
+						}
+					}
+				}
+				/*if only one realm active, return it*/
+				if(p == 1) {
+					for (p = 1; p < i; p++) {
+						realmptr = rr_array[p];
+						if(realmptr->active == TRUE) {
+							return realmptr;
+						}
+					}
+				}
+				for(p = 1; p < i; p++) {
+					realmptr = rr_array[p];
+					if(realmptr->chose == 1) {
+						efform++;
+					}
+				}
+
+				/*    
+                                 *    if none available, 
+				 *    and last realm has
+				 *    been returned as access-reply
+                                 */
+
+				if(efform == cl->total && proxy_return == 0) {
+					for(p = 1; p < i; p++) {
+						realmptr = rr_array[p];
+						realmptr->chose = 0;
+					}
+				}
+
+				/*    
+                                 *    if this realm hasn't been chosen and 
+				 *    this is not being returned for
+                                 *    access reply
+				 *    else if the realm has been chosen 
+                                 *    and this is for access-reply
+                                 *    else this realm is dead (reset if
+				 *    this is the last realm in the list)
+				 */
+
+                                for(p = 1; p < i; p++) {
+					realmptr = rr_array[p];
+					if((realmptr->chose == 0) &&
+                                           (proxy_return < realmptr->node) &&
+					   (realmptr->active == TRUE && realmptr->acct_active == TRUE)
+                                        ) {
+						proxy_return = realmptr->node;
+						realmptr->chose = 1;
+						return realmptr;
+					}
+					else if ((realmptr->chose == 1) &&
+						 (proxy_return == realmptr->node)
+                                        ) {
+						if(proxy_return + 1 > realmptr->total) {
+							proxy_return = 0;
+						}
+						realmptr->chose = 0;
+						return realmptr;
+					}
+					else 
+					{
+						if((realmptr->active == FALSE ||
+						    realmptr->acct_active == FALSE ) &&
+						   (realmptr->node == realmptr->total)
+                                                ) {
+							proxy_return = 0;
+						}
+						continue;
+					}
+				}
+			}
+			
+			/*    
+			 *    else we have single 
+                         *    realm or non-rr realm
+                         */
+			
+			else {
+				return cl;
+			}
 		}
 
 		/*
@@ -589,7 +687,7 @@ REALM *realm_find(const char *realm, int acct)
 	 *	to NOT fall through to the DEFAULT realm.  Therefore,
 	 *	we return NULL, which means "no match found".
 	 */
-	if (!proxy_fallback && dead_match) {
+	if (!mainconfig.proxy_fallback && dead_match) {
 		return NULL;
 	}
 
@@ -616,7 +714,7 @@ REALM *realm_findbyaddr(uint32_t ipaddr, int port)
 	 *	If we get a packet from an end server, then we mark it
 	 *	as active, and return the realm.
 	 */
-	for(cl = realms; cl != NULL; cl = cl->next)
+	for(cl = mainconfig.realms; cl != NULL; cl = cl->next)
 		if ((ipaddr == cl->ipaddr) && (port == cl->auth_port)) {
 			cl->active = TRUE;
 			return cl;
@@ -627,3 +725,40 @@ REALM *realm_findbyaddr(uint32_t ipaddr, int port)
 
 	return NULL;
 }
+
+/*
+ *    Catch any errors in set-up of RR list or exceeding max_proxies
+ */
+void check_proxies(int max_config) {
+	REALM *next, *rptr;
+	if(max_config > 0) {
+		maximum_proxies = max_config;
+		for(rptr=mainconfig.realms; rptr; rptr = rptr->next) {
+			if(rptr->total > max_config || rptr->total >= 1) {
+				for(next = mainconfig.realms; next; next = next->next) {
+					if(next->ldflag != 0 &&
+                                           next->ldflag != 1) {
+						radlog(L_ERR,
+						    "Invalid load balance flag on realm: %s entry %d\n",
+                                                     next->realm, next->node);
+						exit(1);
+					}
+					else if((strcasecmp(rptr->realm, next->realm) == 0) && 
+                                            rptr->ldflag != next->ldflag){ 
+                                               radlog(L_ERR,
+                                                   "Inconsistent ldflag for realm: %s entries %d and %d.\n", 
+                                                    rptr->realm, rptr->node, next->node);
+					       exit(1);
+					}
+				        else if (next->total > max_config) {
+				            radlog(L_ERR,
+				                "Too many entries (%d) for realm: %s. Max is %d.\n",
+ 				 	         next->total, next->realm, max_config);
+				            exit(1);
+				        }
+				}
+			}
+		}
+		maximum_proxies++;
+	}
+}
