@@ -1,7 +1,7 @@
 /*
  * log.c	Logging module.
  *
- * Version:	$Id: log.c,v 1.34 2002/10/29 22:55:27 aland Exp $
+ * Version:	$Id: log.c,v 1.34.2.1 2002/12/09 18:47:06 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@
  * Copyright 2001  Chad Miller <cmiller@surfsouth.com>
  */
 
-static const char rcsid[] = "$Id: log.c,v 1.34 2002/10/29 22:55:27 aland Exp $";
+static const char rcsid[] = "$Id: log.c,v 1.34.2.1 2002/12/09 18:47:06 aland Exp $";
 
 #include "autoconf.h"
 #include "libradius.h"
@@ -73,7 +73,7 @@ int vradlog(int lvl, const char *fmt, va_list ap)
 
 		if ((msgfd = fopen(mainconfig.log_file, "a")) == NULL) {
 		         fprintf(stderr, "%s: Couldn't open %s for logging: %s\n",
-				 progname, buffer, strerror(errno));
+				 progname, mainconfig.log_file, strerror(errno));
 				
 			 fprintf(stderr, "  (");
 			 vfprintf(stderr, fmt, ap);  /* the message that caused the log */
