@@ -1,7 +1,7 @@
 /* 
  * smux.c	SNMP support
  *
- * Version:	$Id: smux.c,v 1.11 2001/03/05 03:42:11 cmiller Exp $
+ * Version:	$Id: smux.c,v 1.12 2002/06/21 19:57:27 cparker Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@
  */
 
 static const char rcsid[] =
-"$Id: smux.c,v 1.11 2001/03/05 03:42:11 cmiller Exp $";
+"$Id: smux.c,v 1.12 2002/06/21 19:57:27 cparker Exp $";
 
 #include "autoconf.h"
 
@@ -1021,10 +1021,10 @@ smux_str2oid (char *str, oid *my_oid, size_t *oid_len)
 		return 0;
 
 	while (1) {
-		if (! isdigit (*str))
+		if (! isdigit ((int) *str))
 			return -1;
 
-		while (isdigit (*str)) {
+		while (isdigit ((int) *str)) {
 			val *= 10;
 			val += (*str - '0');
 			str++;
