@@ -1,7 +1,7 @@
 /*
  * rlm_counter.c
  *
- * Version:  $Id: rlm_counter.c,v 1.21 2002/06/21 19:57:27 cparker Exp $
+ * Version:  $Id: rlm_counter.c,v 1.27 2002/09/06 11:45:38 kkalev Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
  *
  * Copyright 2001  The FreeRADIUS server project
  * Copyright 2001  Alan DeKok <aland@ox.org>
- * Copyright 2001  Kostas Kalevras <kkalev@noc.ntua.gr>
+ * Copyright 2001,2002  Kostas Kalevras <kkalev@noc.ntua.gr>
  */
 
 #include "config.h"
@@ -54,7 +54,7 @@
 #define gdbm_fdesc(foo) (-1)
 #endif
 
-static const char rcsid[] = "$Id: rlm_counter.c,v 1.21 2002/06/21 19:57:27 cparker Exp $";
+static const char rcsid[] = "$Id: rlm_counter.c,v 1.27 2002/09/06 11:45:38 kkalev Exp $";
 
 /*
  *	Define a structure for our module configuration.
@@ -64,22 +64,22 @@ static const char rcsid[] = "$Id: rlm_counter.c,v 1.21 2002/06/21 19:57:27 cpark
  *	be used as the instance handle.
  */
 typedef struct rlm_counter_t {
-	char *filename;  /* name of the database file */
-	char *reset;  /* daily, weekly, monthly, never or user defined */
-	char *key_name;  /* User-Name */
-	char *count_attribute;  /* Acct-Session-Time */
-	char *counter_name;  /* Daily-Session-Time */
-	char *check_name;  /* Daily-Max-Session */
-	char *service_type;  /* Service-Type to search for */
+	char *filename;		/* name of the database file */
+	char *reset;		/* daily, weekly, monthly, never or user defined */
+	char *key_name;		/* User-Name */
+	char *count_attribute;	/* Acct-Session-Time */
+	char *counter_name;	/* Daily-Session-Time */
+	char *check_name;	/* Daily-Max-Session */
+	char *service_type;	/* Service-Type to search for */
 	int cache_size;
 	int service_val;
 	int key_attr;
 	int count_attr;
-	time_t reset_time;
-	time_t last_reset;
-	int dict_attr;  /* attribute number for the counter. */
-	GDBM_FILE gdbm;
-	int fd;
+	time_t reset_time;	/* The time of the next reset. */
+	time_t last_reset;	/* The time of the last reset. */
+	int dict_attr;		/* attribute number for the counter. */
+	GDBM_FILE gdbm;		/* The gdbm file handle */
+	pthread_mutex_t mutex;	/* A mutex to lock the gdbm file for only one reader/writer */
 } rlm_counter_t;
 
 /*
@@ -141,6 +141,76 @@ static int counter_cmp(void *instance, REQUEST *req, VALUE_PAIR *request, VALUE_
 	return counter - check->lvalue;
 }
 
+static int add_defaults(rlm_counter_t *data)
+{
+	datum key_datum;
+	datum time_datum;
+	const char *default1 = "DEFAULT1";
+	const char *default2 = "DEFAULT2";
+	
+	DEBUG2("rlm_counter: add_defaults: Start");
+
+	key_datum.dptr = (const char *) default1;
+	key_datum.dsize = strlen(default1);
+	time_datum.dptr = (char *) &data->reset_time;
+	time_datum.dsize = sizeof(time_t);
+
+	if (gdbm_store(data->gdbm, key_datum, time_datum, GDBM_REPLACE) < 0){
+		radlog(L_ERR, "rlm_counter: Failed storing data to %s: %s",
+				data->filename, gdbm_strerror(gdbm_errno));
+		return RLM_MODULE_FAIL;
+	}
+	DEBUG2("rlm_counter: DEFAULT1 set to %d",(int)data->reset_time);
+
+	key_datum.dptr = (const char *) default2;
+	key_datum.dsize = strlen(default2);
+	time_datum.dptr = (char *) &data->last_reset;
+	time_datum.dsize = sizeof(time_t);
+
+	if (gdbm_store(data->gdbm, key_datum, time_datum, GDBM_REPLACE) < 0){
+		radlog(L_ERR, "rlm_counter: Failed storing data to %s: %s",
+				data->filename, gdbm_strerror(gdbm_errno));
+		return RLM_MODULE_FAIL;
+	}
+	DEBUG2("rlm_counter: DEFAULT2 set to %d",(int)data->last_reset);
+	DEBUG2("rlm_counter: add_defaults: End");
+
+	return RLM_MODULE_OK;
+}
+
+static int reset_db(rlm_counter_t *data)
+{
+	int cache_size = data->cache_size;
+	int ret;
+
+	DEBUG2("rlm_counter: reset_db: Closing database");
+	gdbm_close(data->gdbm);
+
+	/*
+	 *	Open a completely new database.
+	 */
+	data->gdbm = gdbm_open(data->filename, sizeof(int),
+			GDBM_NEWDB | GDBM_COUNTER_OPTS, 0600, NULL);
+	if (data->gdbm == NULL) {
+		radlog(L_ERR, "rlm_counter: Failed to open file %s: %s",
+				data->filename, strerror(errno));
+		return RLM_MODULE_FAIL;
+	}
+	if (gdbm_setopt(data->gdbm, GDBM_CACHESIZE, &cache_size, sizeof(int)) == -1)
+		radlog(L_ERR, "rlm_counter: Failed to set cache size");
+	DEBUG2("rlm_counter: reset_db: Opened new database");
+
+	/*
+	 * Add defaults
+	 */
+	ret = add_defaults(data);
+	if (ret != RLM_MODULE_OK)
+		return ret;	
+
+	DEBUG2("rlm_counter: reset_db ended");
+
+	return RLM_MODULE_OK;
+}
 
 static int find_next_reset(rlm_counter_t *data, time_t timeval)
 {
@@ -223,6 +293,11 @@ static int counter_instantiate(CONF_SECTION *conf, void **instance)
 	ATTR_FLAGS flags;
 	time_t now;
 	int cache_size;
+	int ret;
+	datum key_datum;
+	datum time_datum;
+	const char *default1 = "DEFAULT1";
+	const char *default2 = "DEFAULT2";
 	
 	/*
 	 *	Set up a storage area for instance data
@@ -317,7 +392,7 @@ static int counter_instantiate(CONF_SECTION *conf, void **instance)
 	}	
 
 	/*
-	 *  Discover when next to reset the database.
+	 * Find when to reset the database.
 	 */
 	if (data->reset == NULL) {
 		radlog(L_ERR, "rlm_counter: 'reset' must be set.");
@@ -325,6 +400,7 @@ static int counter_instantiate(CONF_SECTION *conf, void **instance)
 	}
 	now = time(NULL);
 	data->reset_time = 0;
+	data->last_reset = now;
 
 	if (find_next_reset(data,now) == -1)
 		return -1;
@@ -340,17 +416,66 @@ static int counter_instantiate(CONF_SECTION *conf, void **instance)
 				data->filename, strerror(errno));
 		return -1;
 	}
-	if (data->fd >= 0) data->fd = gdbm_fdesc(data->gdbm);
-
 	if (gdbm_setopt(data->gdbm, GDBM_CACHESIZE, &cache_size, sizeof(int)) == -1)
 		radlog(L_ERR, "rlm_counter: Failed to set cache size");
 
+	/*
+	 * Look for the DEFAULT1 entry. This entry if it exists contains the
+	 * time of the next database reset. This time is set each time we reset
+	 * the database. If next_reset < now then we reset the database.
+	 * That way we can overcome the problem where radiusd is down during a database
+	 * reset time. If we did not keep state information in the database then the reset
+	 * would be extended and that would create problems.
+	 *
+	 * We also store the time of the last reset in the DEFAULT2 entry.
+	 *
+	 * If DEFAULT1 and DEFAULT2 do not exist (new database) we add them to the database
+	 */
+
+	key_datum.dptr = (const char *)default1;
+	key_datum.dsize = strlen(default1);
+
+	time_datum = gdbm_fetch(data->gdbm, key_datum);
+	if (time_datum.dptr != NULL){
+		time_t next_reset = 0;
+
+		memcpy(&next_reset, time_datum.dptr, sizeof(time_t));
+		free(time_datum.dptr);
+		if (next_reset <= now){
+
+			data->last_reset = now;
+			ret = reset_db(data);
+			if (ret != RLM_MODULE_OK)
+				return -1;
+		}
+		else
+			data->reset_time = next_reset;
+		key_datum.dptr = (const char *)default2;
+		key_datum.dsize = strlen(default2);
+
+		time_datum = gdbm_fetch(data->gdbm, key_datum);	
+		if (time_datum.dptr != NULL){
+			memcpy(&data->last_reset, time_datum.dptr, sizeof(time_t));
+			free(time_datum.dptr);
+		}
+	}
+	else{
+		ret = add_defaults(data);
+		if (ret != RLM_MODULE_OK)
+			return -1;
+	}
+
 
 	/*
 	 *	Register the counter comparison operation.
 	 */
 	paircompare_register(data->dict_attr, 0, counter_cmp, data);
 
+	/*
+	 * Init the mutex
+	 */
+	pthread_mutex_init(&data->mutex, NULL);
+
 	*instance = data;
 	
 	return 0;
@@ -374,29 +499,15 @@ static int counter_accounting(void *instance, REQUEST *request)
 	 *	the counters.
 	 */
 	if (data->reset_time && (data->reset_time <= request->timestamp)) {
-		int cache_size = data->cache_size;
+		int ret;
 
-		gdbm_close(data->gdbm);
-
-		/*
-		 *	Re-set the next time to clean the database.
-		 */
 		data->last_reset = data->reset_time;
 		find_next_reset(data,request->timestamp);
-
-		/*
-		 *	Open a completely new database.
-		 */
-		data->gdbm = gdbm_open(data->filename, sizeof(int),
-				GDBM_NEWDB | GDBM_COUNTER_OPTS, 0600, NULL);
-		if (data->gdbm == NULL) {
-			radlog(L_ERR, "rlm_counter: Failed to open file %s: %s",
-					data->filename, strerror(errno));
-			return RLM_MODULE_FAIL;
-		}
-		if (data->fd >= 0) data->fd = gdbm_fdesc(data->gdbm);
-		if (gdbm_setopt(data->gdbm, GDBM_CACHESIZE, &cache_size, sizeof(int)) == -1)
-			radlog(L_ERR, "rlm_counter: Failed to set cache size");
+		pthread_mutex_lock(&data->mutex);
+		ret = reset_db(data);
+		pthread_mutex_unlock(&data->mutex);
+		if (ret != RLM_MODULE_OK)
+			return ret;
 	}
 	/*
 	 * Check if we need to watch out for a specific service-type. If yes then check it
@@ -428,7 +539,9 @@ static int counter_accounting(void *instance, REQUEST *request)
 	key_datum.dptr = key_vp->strvalue;
 	key_datum.dsize = key_vp->length;
 
+	pthread_mutex_lock(&data->mutex);
 	count_datum = gdbm_fetch(data->gdbm, key_datum);
+	pthread_mutex_unlock(&data->mutex);
 	if (count_datum.dptr == NULL)
 		counter = 0;
 	else{
@@ -436,7 +549,7 @@ static int counter_accounting(void *instance, REQUEST *request)
 		free(count_datum.dptr);
 	}
 
-	if (count_vp->type == PW_TYPE_DATE) {
+	if (data->count_attr == PW_ACCT_SESSION_TIME) {
 		/*
 		 *	If session time < diff then the user got in after the
 		 *	last reset. So add his session time, otherwise add the
@@ -467,7 +580,9 @@ static int counter_accounting(void *instance, REQUEST *request)
 	count_datum.dptr = (char *) &counter;
 	count_datum.dsize = sizeof(int);
 
+	pthread_mutex_lock(&data->mutex);
 	rcode = gdbm_store(data->gdbm, key_datum, count_datum, GDBM_REPLACE);
+	pthread_mutex_unlock(&data->mutex);
 	if (rcode < 0) {
 		radlog(L_ERR, "rlm_counter: Failed storing data to %s: %s",
 				data->filename, gdbm_strerror(gdbm_errno));
@@ -505,29 +620,15 @@ static int counter_authorize(void *instance, REQUEST *request)
 	 *	the counters.
 	 */
 	if (data->reset_time && (data->reset_time <= request->timestamp)) {
-		int cache_size = data->cache_size;
+		int ret2;
 
-		gdbm_close(data->gdbm);
-
-		/*
-		 *	Re-set the next time to clean the database.
-		 */
 		data->last_reset = data->reset_time;
 		find_next_reset(data,request->timestamp);
-
-		/*
-		 *	Open a completely new database.
-		 */
-		data->gdbm = gdbm_open(data->filename, sizeof(int),
-				GDBM_NEWDB | GDBM_COUNTER_OPTS, 0600, NULL);
-		if (data->gdbm == NULL) {
-			radlog(L_ERR, "rlm_counter: Failed to open file %s: %s",
-					data->filename, strerror(errno));
-			return RLM_MODULE_FAIL;
-		}
-		data->fd = gdbm_fdesc(data->gdbm);
-		if (gdbm_setopt(data->gdbm, GDBM_CACHESIZE, &cache_size, sizeof(int)) == -1)
-			radlog(L_ERR, "rlm_counter: Failed to set cache size");
+		pthread_mutex_lock(&data->mutex);
+		ret2 = reset_db(data);
+		pthread_mutex_unlock(&data->mutex);
+		if (ret2 != RLM_MODULE_OK)
+			return ret2;
 	}
 
 
@@ -556,7 +657,9 @@ static int counter_authorize(void *instance, REQUEST *request)
 	key_datum.dptr = key_vp->strvalue;
 	key_datum.dsize = key_vp->length;
 	
+	pthread_mutex_lock(&data->mutex);
 	count_datum = gdbm_fetch(data->gdbm, key_datum);
+	pthread_mutex_unlock(&data->mutex);
 	if (count_datum.dptr != NULL){
 		memcpy(&counter, count_datum.dptr, sizeof(int));
 		free(count_datum.dptr);
@@ -567,43 +670,50 @@ static int counter_authorize(void *instance, REQUEST *request)
 	 */
 	res=check_vp->lvalue - counter;
 	if (res > 0) {
-		/*
-		 *	We are assuming that simultaneous-use=1. But
-		 *	even if that does not happen then our user
-		 *	could login at max for 2*max-usage-time Is
-		 *	that acceptable?
-		 */
+		if (data->count_attr == PW_ACCT_SESSION_TIME) {
+			/*
+			 * Do the following only if the count attribute is 
+			 * AcctSessionTime
+			 */
 
-		/*
-		 *	User is allowed, but set Session-Timeout.
-		 *	Stolen from main/auth.c
-		 */
+			/*
+		 	*	We are assuming that simultaneous-use=1. But
+		 	*	even if that does not happen then our user
+		 	*	could login at max for 2*max-usage-time Is
+		 	*	that acceptable?
+		 	*/
 
-		/*
-		 *	If we are near a reset then add the next
-		 *	limit, so that the user will not need to
-		 *	login again
-		 */
-		if (data->reset_time && (
-			res >= (data->reset_time - request->timestamp))) {
-			res += check_vp->lvalue;
-		}
+			/*
+		 	*	User is allowed, but set Session-Timeout.
+		 	*	Stolen from main/auth.c
+		 	*/
 
-		DEBUG2("rlm_counter: (Check item - counter) is greater than zero");
-		if ((reply_item = pairfind(request->reply->vps, PW_SESSION_TIMEOUT)) != NULL) {
-			if (reply_item->lvalue > res)
+			/*
+		 	*	If we are near a reset then add the next
+		 	*	limit, so that the user will not need to
+		 	*	login again
+		 	*/
+			if (data->reset_time && (
+				res >= (data->reset_time - request->timestamp))) {
+				res += check_vp->lvalue;
+			}
+
+			if ((reply_item = pairfind(request->reply->vps, PW_SESSION_TIMEOUT)) != NULL) {
+				if (reply_item->lvalue > res)
+					reply_item->lvalue = res;
+			} else {
+				if ((reply_item = paircreate(PW_SESSION_TIMEOUT, PW_TYPE_INTEGER)) == NULL) {
+					radlog(L_ERR|L_CONS, "no memory");
+					return RLM_MODULE_NOOP;
+				}
 				reply_item->lvalue = res;
-		} else {
-			if ((reply_item = paircreate(PW_SESSION_TIMEOUT, PW_TYPE_INTEGER)) == NULL) {
-				radlog(L_ERR|L_CONS, "no memory");
-				return RLM_MODULE_NOOP;
+				pairadd(&request->reply->vps, reply_item);
 			}
-			reply_item->lvalue = res;
-			pairadd(&request->reply->vps, reply_item);
 		}
 
 		ret=RLM_MODULE_OK;
 
+		DEBUG2("rlm_counter: (Check item - counter) is greater than zero");
 		DEBUG2("rlm_counter: Authorized user %s, check_item=%d, counter=%d",
 				key_vp->strvalue,check_vp->lvalue,counter);
 		DEBUG2("rlm_counter: Sent Reply-Item for user %s, Type=Session-Timeout, value=%d",
@@ -644,6 +754,7 @@ static int counter_detach(void *instance)
 	free(data->key_name);
 	free(data->count_attribute);
 	free(data->counter_name);
+	pthread_mutex_destroy(&data->mutex);
 
 	free(instance);
 	return 0;
@@ -660,7 +771,7 @@ static int counter_detach(void *instance)
  */
 module_t rlm_counter = {
 	"Counter",	
-	RLM_TYPE_THREAD_UNSAFE,		/* type */
+	RLM_TYPE_THREAD_SAFE,		/* type */
 	NULL,				/* initialization */
 	counter_instantiate,		/* instantiation */
 	{
@@ -668,7 +779,10 @@ module_t rlm_counter = {
 		counter_authorize, 	/* authorization */
 		NULL,			/* preaccounting */
 		counter_accounting,	/* accounting */
-		NULL			/* checksimul */
+		NULL,			/* checksimul */
+		NULL,			/* pre-proxy */
+		NULL,			/* post-proxy */
+		NULL			/* post-auth */
 	},
 	counter_detach,			/* detach */
 	NULL,				/* destroy */
