@@ -1,7 +1,7 @@
 /*
  * rlm_detail.c	accounting:    Write the "detail" files.
  *
- * Version:	$Id: rlm_detail.c,v 1.27 2002/07/08 18:23:42 aland Exp $
+ * Version:	$Id: rlm_detail.c,v 1.28 2002/08/24 16:54:57 aland Exp $
  *
  *  This program is is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License, version 2 if the
@@ -19,7 +19,7 @@
  * Copyright 2000  The FreeRADIUS server project
  */
 
-static const char rcsid[] = "$Id: rlm_detail.c,v 1.27 2002/07/08 18:23:42 aland Exp $";
+static const char rcsid[] = "$Id: rlm_detail.c,v 1.28 2002/08/24 16:54:57 aland Exp $";
 
 #include	"autoconf.h"
 #include	"libradius.h"
@@ -298,7 +298,10 @@ module_t rlm_detail = {
 		NULL,		 	/* authorization */
 		NULL,			/* preaccounting */
 		detail_accounting,	/* accounting */
-		NULL			/* checksimul */
+		NULL,			/* checksimul */
+		NULL,			/* pre-proxy */
+		NULL,			/* post-proxy */
+		NULL			/* post-auth */
 	},
 	detail_detach,			/* detach */
 	NULL				/* destroy */
