@@ -1,7 +1,7 @@
 /*
  * eap_tls.c
  *
- * Version:     $Id: eap_tls.c,v 1.9 2002/07/11 17:02:22 raghu Exp $
+ * Version:     $Id: eap_tls.c,v 1.11 2002/09/17 13:33:50 cparker Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -82,7 +82,6 @@ int eaptls_start(EAP_DS *eap_ds)
 	EAPTLS_PACKET 	reply;
 
 	reply.code = EAPTLS_START;
-	reply.id = eap_ds->response->id + 1/*Incrementor*/;
 	reply.length = TLS_HEADER_LEN + 1/*flags*/;
 
 	reply.flags = 0x00;
@@ -101,7 +100,6 @@ int eaptls_success(EAP_DS *eap_ds)
 	EAPTLS_PACKET	reply;
 
 	reply.code = EAPTLS_SUCCESS;
-	reply.id = eap_ds->response->id + 1;
 	reply.length = TLS_HEADER_LEN;
 	reply.flags = 0x00;
 	reply.data = NULL;
@@ -117,7 +115,6 @@ int eaptls_fail(EAP_DS *eap_ds)
 	EAPTLS_PACKET	reply;
 
 	reply.code = EAPTLS_FAIL;
-	reply.id = eap_ds->response->id + 1;
 	reply.length = TLS_HEADER_LEN;
 	reply.flags = 0x00;
 	reply.data = NULL;
@@ -170,7 +167,6 @@ int eaptls_request(EAP_DS *eap_ds, tls_session_t *ssn)
 	}
 
 	reply.code = EAPTLS_REQUEST;
-	reply.id = eap_ds->response->id + 1;
 	reply.flags = 0x00;
 
 	/* Send data, NOT more than the FRAGMENT size */
@@ -234,6 +230,8 @@ eaptls_status_t eaptls_ack_handler(EAP_HANDLER *handler)
 	case handshake:
 		if (tls_session->info.handshake_type == finished) {
 			eaptls_success(handler->eap_ds);
+			eaptls_gen_mppe_keys(handler->reply_vps, 
+					     tls_session->ssl);
 			session_free(&handler->opaque);
 			return EAPTLS_SUCCESS;
 		} else if (tls_session->fragment > 0) {
@@ -272,7 +270,6 @@ int eaptls_send_ack(EAP_DS *eap_ds)
 	EAPTLS_PACKET 	reply;
 
 	reply.code = EAPTLS_ACK;
-	reply.id = eap_ds->response->id + 1;
 	reply.length = TLS_HEADER_LEN + 1/*flags*/;
 	reply.flags = 0x00;
 	reply.data = NULL;
@@ -501,8 +498,6 @@ int eaptls_compose(EAP_DS *eap_ds, EAPTLS_PACKET *reply)
    Identifier value in the subsequent fragment contained within an EAP-
    Reponse.
 */
-	eap_ds->request->id = (reply->id)?(reply->id):eap_ds->response->id + 1;
-
 	eap_ds->request->type.data = malloc(reply->length - TLS_HEADER_LEN);
 	if (eap_ds->request->type.data == NULL) {
 		radlog(L_ERR, "rlm_eap_tls: out of memory");
