@@ -2,7 +2,7 @@
  * rlm_fastusers.c	authorization: Find a user in the hashed "users" file.
  *                	accounting:    Do nothing.  Auth module only.
  *
- * Version:	$Id: rlm_fastusers.c,v 1.25 2001/11/09 00:53:10 kkalev Exp $
+ * Version:	$Id: rlm_fastusers.c,v 1.26 2002/08/24 16:54:57 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -823,7 +823,10 @@ module_t rlm_fastusers = {
 		fastuser_authorize,	/* authorization */
 		fastuser_preacct,	/* preaccounting */
 		fastuser_accounting,	/* accounting */
-		NULL			/* checksimul */
+		NULL,			/* checksimul */
+		NULL,			/* pre-proxy */
+		NULL,			/* post-proxy */
+		NULL			/* post-auth */
 	},
 	fastuser_detach,		/* detach */
 	NULL				/* destroy */
