@@ -59,7 +59,7 @@
  *	  collisions with other modules
  *	- If perform_search fails check the ld != NULL before using it. Based on a bug report
  *	  by John <jhogenmiller@pennswoods.net>
- * June 2002, Kostas Kalevras <kkalev@noc.ntua.gr>
+ * Jun 2002, Kostas Kalevras <kkalev@noc.ntua.gr>
  *	- Add the ability to do a paircmp on the check items. Add a compare_check_items boolean
  *	  configuration directive which defaults to no. If it is set then we will do a compare
  *	- Add another configuration directive. access_attr_used_for_allow. If it is set to yes
@@ -71,7 +71,7 @@
  *	  we are using is of type fast and can deadlock if the same thread tries to relock it. That
  *	  could happen in case of calls to xlat.
  *	- When ldap_search returns NO_SUCH_OBJECT don't return fail but notfound
- * July 2002, Kostas Kalevras <kkalev@noc.ntua.gr>
+ * Jul 2002, Kostas Kalevras <kkalev@noc.ntua.gr>
  *	- Fix the logic when we get an LDAP_SERVER_DOWN or we have conn->ld == NULL in perform_search
  *	- Try to minimize the penalty of having the ldap server go down. The comments before
  *	  MAX_FAILED_CONNS_* definitions should explain things.
@@ -81,8 +81,30 @@
  *	  group membership combine them in one ldap search operation. That should make group
  *	  membership checks a lot faster.
  *	- Remember to do ldap_release_conn and ldap_msgfree when we do paircmp and the result is reject
+ * Aug 2002, Kostas Kalevras <kkalev@noc.ntua.gr>
+ *	- Add support for group membership attribute inside the user entry in ldap_groupcmp. The attribute
+ *	  can either contain the name or the DN of the group. Added the groupmembership_attribute
+ *	  configuration directive
+ *	- Move the ldap_{get,release}_conn in ldap_groupcmp so that we hold a connection for the minimum time.
+ *	- Now that ldap_groupcmp is complete we really don't need access_group. Removed it.
+ *	- Remember to free groupmembership_attribute in ldap_detach
+ *	- Don't delete existing generic attributes in ldap_pairget when adding new ones. Since generic attributes
+ *	  have operators we don't need to try to be cleaver.
+ * Sep 2002, Kostas Kalevras <kkalev@noc.ntua.gr>
+ *	- Fix a crash in ldap_pairget when the attribute value is larger than the buffer size
+ *	  Bug report by Stefan Radovanovici <sra@rtsffm.com>
+ *	- If we add a check item then use the == operator. Based on an idea by Allister Maguire <amaguire@gnc.net.nz>
+ *	- Only add a failure message for bind as user failed in ldap_authenticate if the result of ldap_connect was
+ *	  RLM_MODULE_REJECT
+ *	- Make tls_mode a configurable option. Patch from John <jhogenmiller@pennswoods.net>
+ *	- Allow multiple regular profiles for an entry
+ * Oct 2002, Kostas Kalevras <kkalev@noc.ntua.gr>
+ *	- Disable cache after searching for the default profile
+ *	- Use the MAX_FAILED_CONNS_* in ldap_authenticate() when calling ldap_connect()
+ * Nov 2002, Kostas Kalevras <kkalev@noc.ntua.gr>
+ *	- Set LDAP version to V3 before binding. Now freeradius should work with openldap21
  */
-static const char rcsid[] = "$Id: rlm_ldap.c,v 1.84 2002/07/18 08:21:20 pam Exp $";
+static const char rcsid[] = "$Id: rlm_ldap.c,v 1.96 2002/11/01 21:36:43 kkalev Exp $";
 
 #include "autoconf.h"
 
@@ -178,19 +200,20 @@ typedef struct {
 	int		cache_size;
 	int		do_comp;
 	int		default_allow;
+	int		failed_conns;
 	char           *login;
 	char           *password;
 	char           *filter;
 	char           *basedn;
 	char           *default_profile;
 	char           *profile_attr;
-	char           *access_group;
 	char           *access_attr;
 	char           *passwd_hdr;
 	char           *passwd_attr;
 	char           *dictionary_mapping;
 	char	       *groupname_attr;
 	char	       *groupmemb_filt;
+	char           *groupmemb_attr;
 	char		**atts;
 	TLDAP_RADIUS   *check_item_map;
 	TLDAP_RADIUS   *reply_item_map;
@@ -212,12 +235,12 @@ static CONF_PARSER module_config[] = {
 	{"ldap_cache_size", PW_TYPE_INTEGER, offsetof(ldap_instance,cache_size), NULL, "0"},
 	{"identity", PW_TYPE_STRING_PTR, offsetof(ldap_instance,login), NULL, ""},
 	{"start_tls", PW_TYPE_BOOLEAN, offsetof(ldap_instance,start_tls), NULL, "no"},
+	{"tls_mode", PW_TYPE_BOOLEAN, offsetof(ldap_instance,tls_mode), NULL, "no"},
 	{"password", PW_TYPE_STRING_PTR, offsetof(ldap_instance,password), NULL, ""},
 	{"basedn", PW_TYPE_STRING_PTR, offsetof(ldap_instance,basedn), NULL, NULL},
 	{"filter", PW_TYPE_STRING_PTR, offsetof(ldap_instance,filter), NULL, "(uid=%u)"},
 	{"default_profile", PW_TYPE_STRING_PTR, offsetof(ldap_instance,default_profile), NULL, NULL},
 	{"profile_attribute", PW_TYPE_STRING_PTR, offsetof(ldap_instance,profile_attr), NULL, NULL},
-	{"access_group", PW_TYPE_STRING_PTR, offsetof(ldap_instance,access_group), NULL, NULL},
 	{"password_header", PW_TYPE_STRING_PTR, offsetof(ldap_instance,passwd_hdr), NULL, NULL},
 	{"password_attribute", PW_TYPE_STRING_PTR, offsetof(ldap_instance,passwd_attr), NULL, NULL},
 	/* LDAP attribute name that controls remote access */
@@ -225,6 +248,7 @@ static CONF_PARSER module_config[] = {
 	/* file with mapping between LDAP and RADIUS attributes */
 	{"groupname_attribute", PW_TYPE_STRING_PTR, offsetof(ldap_instance,groupname_attr), NULL, "cn"},
 	{"groupmembership_filter", PW_TYPE_STRING_PTR, offsetof(ldap_instance,groupmemb_filt), NULL, "(|(&(objectClass=GroupOfNames)(member=%{Ldap-UserDn}))(&(objectClass=GroupOfUniqueNames)(uniquemember=%{Ldap-UserDn})))"},
+	{"groupmembership_attribute", PW_TYPE_STRING_PTR, offsetof(ldap_instance,groupmemb_attr), NULL, NULL},
 	{"dictionary_mapping", PW_TYPE_STRING_PTR, offsetof(ldap_instance,dictionary_mapping), NULL, "${confdir}/ldap.attrmap"},
 	{"ldap_debug", PW_TYPE_INTEGER, offsetof(ldap_instance,ldap_debug), NULL, "0x0000"},
 	{"ldap_connections_number", PW_TYPE_INTEGER, offsetof(ldap_instance,num_conns), NULL, "5"},
@@ -241,7 +265,7 @@ static CONF_PARSER module_config[] = {
 #ifdef FIELDCPY
 static void     fieldcpy(char *, char **);
 #endif
-static VALUE_PAIR *ldap_pairget(LDAP *, LDAPMessage *, TLDAP_RADIUS *,VALUE_PAIR **);
+static VALUE_PAIR *ldap_pairget(LDAP *, LDAPMessage *, TLDAP_RADIUS *,VALUE_PAIR **,char);
 static int ldap_groupcmp(void *, REQUEST *, VALUE_PAIR *, VALUE_PAIR *, VALUE_PAIR *, VALUE_PAIR **);
 static int ldap_xlat(void *,REQUEST *, char *, char *,int, RADIUS_ESCAPE_STRING);
 static LDAP    *ldap_connect(void *instance, const char *, const char *, int, int *);
@@ -309,11 +333,12 @@ ldap_instantiate(CONF_SECTION * conf, void **instance)
 	/* workaround for servers which support LDAPS but not START TLS */
 	if(inst->port == LDAPS_PORT)
 		inst->tls_mode = LDAP_OPT_X_TLS_HARD;
-	 else
+	else if (inst->tls_mode)
   		inst->tls_mode = LDAP_OPT_X_TLS_TRY;
 	inst->reply_item_map = NULL;
 	inst->check_item_map = NULL;
 	inst->conns = NULL;
+	inst->failed_conns = 0;
 
 	paircompare_register(PW_LDAP_GROUP, PW_USER_NAME, ldap_groupcmp, inst);
 	DEBUG("conns: %p",inst->conns);
@@ -670,6 +695,7 @@ static int ldap_groupcmp(void *instance, REQUEST *req, VALUE_PAIR *request, VALU
                         ldap_msgfree(result);
                         return 1;
                 }
+		ldap_release_conn(conn_id,inst->conns);
                 /*
                 * Adding new attribute containing DN for LDAP object associated with
                 * given username
@@ -681,30 +707,109 @@ static int ldap_groupcmp(void *instance, REQUEST *req, VALUE_PAIR *request, VALU
 
         if(!radius_xlat(gr_filter, MAX_GROUP_STR_LEN, inst->groupmemb_filt, req, NULL)){
                 DEBUG("rlm_ldap::ldap_groupcmp: unable to create filter.");
-		if (conn_id != -1)
-			ldap_release_conn(conn_id,inst->conns);
                 return 1;
         }
-	if (conn_id == -1 && (conn_id = ldap_get_conn(inst->conns,&conn,inst)) == -1){
+
+	snprintf(filter,MAX_GROUP_STR_LEN - 1, "(&(%s=%s)%s)",inst->groupname_attr,(char *)check->strvalue,gr_filter);
+
+	if ((conn_id = ldap_get_conn(inst->conns,&conn,inst)) == -1){
 		radlog(L_ERR, "rlm_ldap: All ldap connections are in use");
 		return 1;
 	}
 
-	snprintf(filter,MAX_GROUP_STR_LEN - 1, "(&(%s=%s)%s)",inst->groupname_attr,(char *)check->strvalue,gr_filter);
-
         if ((res = perform_search(inst, conn, basedn, LDAP_SCOPE_SUBTREE, filter, attrs, &result)) != RLM_MODULE_OK){
-                if (res == RLM_MODULE_NOTFOUND){
-                        DEBUG("rlm_ldap::ldap_groupcmp: Group %s not found or user is not a member", (char *)check->strvalue);
+                if (res != RLM_MODULE_NOTFOUND){
+                	DEBUG("rlm_ldap::ldap_groupcmp: Search returned error");
 			ldap_release_conn(conn_id,inst->conns);
-                        return 1;
+                	return 1;
                 }
-                DEBUG("rlm_ldap::ldap_groupcmp: Search returned error");
-		ldap_release_conn(conn_id,inst->conns);
-                return 1;
         }
+	if (res != RLM_MODULE_NOTFOUND)
+		ldap_msgfree(result);
+	ldap_release_conn(conn_id,inst->conns);
+
+	if (res == RLM_MODULE_NOTFOUND){
+		if (inst->groupmemb_attr == NULL){
+			DEBUG("rlm_ldap::ldap_groupcmp: Group %s not found or user is not a member.",(char *)check->strvalue);
+			return 1;
+		}
+		else{
+			VALUE_PAIR *user_dn;
+			char *group_attrs[] = {inst->groupmemb_attr,NULL};
+
+			user_dn = pairfind(req->packet->vps, PW_LDAP_USERDN);
+			if (user_dn != NULL){
+				char **vals;
+
+				snprintf(filter,MAX_GROUP_STR_LEN - 1, "(objectclass=*)");
+				if ((conn_id = ldap_get_conn(inst->conns,&conn,inst)) == -1){
+					radlog(L_ERR, "rlm_ldap: Add ldap connections are in use");
+					return 1;
+				}
+				if ((res = perform_search(inst, conn, user_dn->strvalue, LDAP_SCOPE_BASE, filter, group_attrs, &result)) != RLM_MODULE_OK){
+					DEBUG("rlm_ldap::ldap_groupcmp: Search returned error");
+					ldap_release_conn(conn_id, inst->conns);
+					return 1;
+				}		
+
+				if ((msg = ldap_first_entry(conn->ld, result)) == NULL) {
+					DEBUG("rlm_ldap::ldap_groupcmp: ldap_first_entry() failed");
+					ldap_release_conn(conn_id,inst->conns);
+					ldap_msgfree(result);
+					return 1;
+				}
+				if ((vals = ldap_get_values(conn->ld, msg, inst->groupmemb_attr)) != NULL) {
+					unsigned int i = 0;
+					char found = 0;
+
+					for (;i < ldap_count_values(vals);i++){
+						if (strchr(vals[i],',') != NULL){ /* This looks like a DN */
+							LDAPMessage *gr_result = NULL;
+
+							snprintf(filter,MAX_GROUP_STR_LEN - 1, "(%s=%s)",inst->groupname_attr,(char *)check->strvalue);
+							if ((res = perform_search(inst, conn, vals[i], LDAP_SCOPE_BASE, filter, attrs, &gr_result)) != RLM_MODULE_OK){
+								if (res != RLM_MODULE_NOTFOUND){
+									DEBUG("rlm_ldap::ldap_groupcmp: Search returned error");
+									ldap_msgfree(result);
+									ldap_value_free(vals);
+									ldap_release_conn(conn_id, inst->conns);
+									return 1;
+								}
+							}
+							else{
+								ldap_msgfree(gr_result);
+								found = 1;
+								break;
+							}
+						}
+						else{
+							if (strcmp(vals[i],(char *)check->strvalue) == 0){
+								found = 1;
+								break;
+							}
+						}
+					}
+					ldap_value_free(vals);
+        				ldap_msgfree(result);
+					if (found == 0){
+						DEBUG("rlm_ldap::groupcmp: Group %s not found or user not a member",
+							(char *)check->strvalue);
+						ldap_release_conn(conn_id,inst->conns);
+						return 1;
+					}
+				}
+				else{
+					DEBUG("rlm_ldap::ldap_groupcmp: ldap_get_values() failed");
+					ldap_release_conn(conn_id,inst->conns);
+					ldap_msgfree(result);
+					return 1;
+				}
+			}
+		}
+	}
+
 
 	DEBUG("rlm_ldap::ldap_groupcmp: User found in group %s",(char *)check->strvalue);
-	ldap_msgfree(result);
 	ldap_release_conn(conn_id,inst->conns);
 
         return 0;
@@ -822,7 +927,6 @@ ldap_authorize(void *instance, REQUEST * request)
 	LDAPMessage	*msg = NULL;
 	LDAPMessage	*def_msg = NULL;
 	LDAPMessage	*def_attr_msg = NULL;
-	LDAPMessage	*gr_result = NULL;
 	LDAPMessage	*def_result = NULL;
 	LDAPMessage	*def_attr_result = NULL;
 	ldap_instance	*inst = instance;
@@ -948,42 +1052,6 @@ ldap_authorize(void *instance, REQUEST * request)
 	if (inst->cache_timeout >0)
 		ldap_enable_cache(conn->ld, inst->cache_timeout, inst->cache_size);
 
-	/* Remote access controled by group membership of the user object */
-	if (inst->access_group != NULL) {
-		static char	group[MAX_AUTH_QUERY_LEN];
-		static char	*attrs[] = {"dn", NULL};
-
-		DEBUG("rlm_ldap: checking user membership in dialup-enabling group %s", inst->access_group);
-		/*
-		 * uniquemember appears in Netscape Directory Server's groups
-		 * since we have objectclass groupOfNames and
-		 * groupOfUniqueNames
-		 */
-		if(!radius_xlat(group, MAX_AUTH_QUERY_LEN, inst->access_group,
-				request, NULL)) 
-			radlog (L_ERR, "rlm_ldap: unable to munge group.\n"); 
-
-		if(!radius_xlat(filter, MAX_AUTH_QUERY_LEN, inst->groupmemb_filt,
-			        request, NULL)) 
-			radlog (L_ERR, "rlm_ldap: unable to create filter.\n"); 
-
-		if ((res = perform_search(instance, conn, group, LDAP_SCOPE_BASE, filter, attrs, &gr_result)) != RLM_MODULE_OK) {
-			if (inst->cache_timeout >0 && conn->ld != NULL)
-				ldap_disable_cache(conn->ld);
-			ldap_msgfree(result);
-			ldap_release_conn(conn_id,inst->conns);
-			if (res == RLM_MODULE_NOTFOUND){
-				snprintf(module_fmsg,sizeof(module_fmsg),"rlm_ldap: User is not an access group member");
-				module_fmsg_vp = pairmake("Module-Failure-Message", module_fmsg, T_OP_EQ);
-				pairadd(&request->packet->vps, module_fmsg_vp);
-				return (RLM_MODULE_USERLOCK);
-			}
-			else
-				return (res);
-		} else 
-			ldap_msgfree(gr_result);
-	}
-
 	/*
 	 * Check for the default profile entry. If it exists then add the 
 	 * attributes it contains in the check and reply pairs
@@ -1001,9 +1069,9 @@ ldap_authorize(void *instance, REQUEST * request)
 				profile, LDAP_SCOPE_BASE, 
 				filter, inst->atts, &def_result)) == RLM_MODULE_OK){
 				if ((def_msg = ldap_first_entry(conn->ld,def_result))){
-					if ((check_tmp = ldap_pairget(conn->ld,def_msg,inst->check_item_map,check_pairs)))
+					if ((check_tmp = ldap_pairget(conn->ld,def_msg,inst->check_item_map,check_pairs,1)))
 						pairadd(check_pairs,check_tmp);
-					if ((reply_tmp = ldap_pairget(conn->ld,def_msg,inst->reply_item_map,reply_pairs)))
+					if ((reply_tmp = ldap_pairget(conn->ld,def_msg,inst->reply_item_map,reply_pairs,0)))
 						pairadd(reply_pairs,reply_tmp);
 				}
 				ldap_msgfree(def_result);
@@ -1011,6 +1079,8 @@ ldap_authorize(void *instance, REQUEST * request)
 				DEBUG("rlm_ldap: default_profile/user-profile search failed");
 		}
 	}
+	if (inst->cache_timeout >0)
+		ldap_disable_cache(conn->ld);
 
 	/*
 	 * Check for the profile attribute. If it exists, we assume that it 
@@ -1020,20 +1090,26 @@ ldap_authorize(void *instance, REQUEST * request)
 	 */
 
 	if (inst->profile_attr){
-		if ((vals = ldap_get_values(conn->ld, msg, inst->profile_attr)) != NULL && strlen(vals[0])) {
+		if ((vals = ldap_get_values(conn->ld, msg, inst->profile_attr)) != NULL) {
+			unsigned int i=0;
 			strncpy(filter,"(objectclass=radiusprofile)",MAX_AUTH_QUERY_LEN);
-			if ((res = perform_search(instance, conn,
-				vals[0], LDAP_SCOPE_BASE, 
-				filter, inst->atts, &def_attr_result)) == RLM_MODULE_OK){
-				if ((def_attr_msg = ldap_first_entry(conn->ld,def_attr_result))){
-					if ((check_tmp = ldap_pairget(conn->ld,def_attr_msg,inst->check_item_map,check_pairs)))
-						pairadd(check_pairs,check_tmp);
-					if ((reply_tmp = ldap_pairget(conn->ld,def_attr_msg,inst->reply_item_map,reply_pairs)))
-						pairadd(reply_pairs,reply_tmp);
-				}
-				ldap_msgfree(def_attr_result);
-			} else 
-				DEBUG("rlm_ldap: profile_attribute search failed");
+			if (inst->cache_timeout >0)
+				ldap_enable_cache(conn->ld, inst->cache_timeout, inst->cache_size);
+			while(vals[i] != NULL && strlen(vals[i])){
+				if ((res = perform_search(instance, conn,
+					vals[i], LDAP_SCOPE_BASE, 
+					filter, inst->atts, &def_attr_result)) == RLM_MODULE_OK){
+					if ((def_attr_msg = ldap_first_entry(conn->ld,def_attr_result))){
+						if ((check_tmp = ldap_pairget(conn->ld,def_attr_msg,inst->check_item_map,check_pairs,1)))
+							pairadd(check_pairs,check_tmp);
+						if ((reply_tmp = ldap_pairget(conn->ld,def_attr_msg,inst->reply_item_map,reply_pairs,0)))
+							pairadd(reply_pairs,reply_tmp);
+					}
+					ldap_msgfree(def_attr_result);
+				} else 
+					DEBUG("rlm_ldap: profile_attribute search failed");
+				i++;
+			}
 			ldap_value_free(vals);
 		}
 	}
@@ -1086,14 +1162,14 @@ ldap_authorize(void *instance, REQUEST * request)
 
 	DEBUG("rlm_ldap: looking for check items in directory...");
 
-	if ((check_tmp = ldap_pairget(conn->ld, msg, inst->check_item_map,check_pairs)) != NULL)
+	if ((check_tmp = ldap_pairget(conn->ld, msg, inst->check_item_map,check_pairs,1)) != NULL)
 		pairadd(check_pairs, check_tmp);
 
 
 	DEBUG("rlm_ldap: looking for reply items in directory...");
 
 
-	if ((reply_tmp = ldap_pairget(conn->ld, msg, inst->reply_item_map,reply_pairs)) != NULL)
+	if ((reply_tmp = ldap_pairget(conn->ld, msg, inst->reply_item_map,reply_pairs,0)) != NULL)
 		pairadd(reply_pairs, reply_tmp);
 
        if (inst->do_comp && paircmp(request,request->packet->vps,*check_pairs,reply_pairs) != 0){
@@ -1147,6 +1223,7 @@ ldap_authenticate(void *instance, REQUEST * request)
 	int		conn_id = -1;
 
 	DEBUG("rlm_ldap: - authenticate");
+
 	/*
 	 * Ensure that we're being passed a plain-text password, and not
 	 * anything else.
@@ -1172,6 +1249,18 @@ ldap_authenticate(void *instance, REQUEST * request)
 		return RLM_MODULE_INVALID;
 	}
 
+	/*
+	 * Check that we don't have any failed connections. If we do there's no real need
+	 * of runing. Also give it another chance if we have a lot of failed connections.
+	 */
+	if (inst->failed_conns > MAX_FAILED_CONNS_END)
+		inst->failed_conns = 0;
+	if (inst->failed_conns > MAX_FAILED_CONNS_START){
+		inst->failed_conns++;
+		return RLM_MODULE_FAIL;
+	}
+
+
 	DEBUG("rlm_ldap: login attempt by \"%s\" with password \"%s\"", 
 	       request->username->strvalue, request->password->strvalue);
 
@@ -1225,15 +1314,23 @@ ldap_authenticate(void *instance, REQUEST * request)
 	ld_user = ldap_connect(instance, user_dn, request->password->strvalue,
 			       1, &res);
 	if (ld_user == NULL){
-		snprintf(module_fmsg,sizeof(module_fmsg),"rlm_ldap: Bind as user failed");
-		module_fmsg_vp = pairmake("Module-Failure-Message", module_fmsg, T_OP_EQ);
-		pairadd(&request->packet->vps, module_fmsg_vp);
+		if (res == RLM_MODULE_REJECT){
+			inst->failed_conns = 0;
+			snprintf(module_fmsg,sizeof(module_fmsg),"rlm_ldap: Bind as user failed");
+			module_fmsg_vp = pairmake("Module-Failure-Message", module_fmsg, T_OP_EQ);
+			pairadd(&request->packet->vps, module_fmsg_vp);
+		}
+		if (res == RLM_MODULE_FAIL){
+			DEBUG("rlm_ldap: ldap_connect() failed");
+			inst->failed_conns++;
+		}
 		return (res);
 	}
 
 	DEBUG("rlm_ldap: user %s authenticated succesfully",
 	      request->username->strvalue);
 	ldap_unbind_s(ld_user);
+	inst->failed_conns = 0;
 
 	return RLM_MODULE_OK;
 }
@@ -1262,6 +1359,10 @@ ldap_connect(void *instance, const char *dn, const char *password, int auth, int
 	if (inst->ldap_debug && ldap_set_option(NULL, LDAP_OPT_DEBUG_LEVEL, &(inst->ldap_debug)) != LDAP_OPT_SUCCESS) {
 		radlog(L_ERR, "rlm_ldap: Could not set LDAP_OPT_DEBUG_LEVEL %d", inst->ldap_debug);
 	}
+	ldap_version = LDAP_VERSION3;
+	if (ldap_set_option(ld, LDAP_OPT_PROTOCOL_VERSION, &ldap_version) != LDAP_OPT_SUCCESS) {
+		radlog(L_ERR, "rlm_ldap: Could not set LDAP version to V3");
+	}
 #ifdef HAVE_LDAP_START_TLS
         if(inst->tls_mode) {
 		DEBUG("rlm_ldap: setting TLS mode to %d", inst->tls_mode);
@@ -1274,17 +1375,14 @@ ldap_connect(void *instance, const char *dn, const char *password, int auth, int
 
 	if (inst->start_tls) {
 		DEBUG("rlm_ldap: starting TLS");
-		ldap_version = LDAP_VERSION3;
-		if (ldap_set_option(ld, LDAP_OPT_PROTOCOL_VERSION, &ldap_version) == LDAP_SUCCESS) {
-			rc = ldap_start_tls_s(ld, NULL, NULL);
-			if (rc != LDAP_SUCCESS) {
-				DEBUG("rlm_ldap: ldap_start_tls_s()");
-				ldap_get_option(ld, LDAP_OPT_ERROR_NUMBER, &ldap_errno);
-				radlog(L_ERR, "rlm_ldap: could not start TLS %s", ldap_err2string(ldap_errno));
-				*result = RLM_MODULE_FAIL;
-				ldap_unbind_s(ld);
-				return (NULL);
-			}
+		rc = ldap_start_tls_s(ld, NULL, NULL);
+		if (rc != LDAP_SUCCESS) {
+			DEBUG("rlm_ldap: ldap_start_tls_s()");
+			ldap_get_option(ld, LDAP_OPT_ERROR_NUMBER, &ldap_errno);
+			radlog(L_ERR, "rlm_ldap: could not start TLS %s", ldap_err2string(ldap_errno));
+			*result = RLM_MODULE_FAIL;
+			ldap_unbind_s(ld);
+			return (NULL);
 		}
 	}
 #endif /* HAVE_LDAP_START_TLS */
@@ -1362,8 +1460,6 @@ ldap_detach(void *instance)
 		free((char *) inst->password);
 	if (inst->basedn)
 		free((char *) inst->basedn);
-	if (inst->access_group)
-		free((char *) inst->access_group);
 	if (inst->dictionary_mapping)
 		free(inst->dictionary_mapping);
 	if (inst->filter)
@@ -1376,6 +1472,8 @@ ldap_detach(void *instance)
 		free((char *) inst->groupname_attr);
 	if (inst->groupmemb_filt)
 		free((char *) inst->groupmemb_filt);
+	if (inst->groupmemb_attr)
+		free((char *) inst->groupmemb_attr);
 	if (inst->conns){
 		int i=0;
 
@@ -1467,7 +1565,7 @@ fieldcpy(char *string, char **uptr)
 
 static VALUE_PAIR *
 ldap_pairget(LDAP * ld, LDAPMessage * entry,
-	     TLDAP_RADIUS * item_map, VALUE_PAIR **pairs)
+	     TLDAP_RADIUS * item_map, VALUE_PAIR **pairs,char is_check)
 {
 	char          **vals;
 	int             vals_count;
@@ -1476,7 +1574,7 @@ ldap_pairget(LDAP * ld, LDAPMessage * entry,
 	TLDAP_RADIUS   *element;
 	LRAD_TOKEN      token;
 	int             is_generic_attribute;
-	char            value[64];
+	char            value[256];
 	VALUE_PAIR     *pairlist = NULL;
 	VALUE_PAIR     *newpair = NULL;
 
@@ -1506,9 +1604,6 @@ ldap_pairget(LDAP * ld, LDAPMessage * entry,
 					if ( (newpair = pairread(&ptr, &dummy)) != NULL) {
 						DEBUG("rlm_ldap: extracted attribute %s from generic item %s", 
 						      newpair->name, vals[vals_idx]);
-						if (! vals_idx){
-							pairdelete(pairs,newpair->attribute);
-						}
 						pairadd(&pairlist, newpair);
 					} else {
 						radlog(L_ERR, "rlm_ldap: parsing %s failed: %s", 
@@ -1516,11 +1611,11 @@ ldap_pairget(LDAP * ld, LDAPMessage * entry,
 					}
 				} else {
 					/* this is a one-to-one-mapped attribute */
-					token = gettoken(&ptr, value, sizeof(value));
+					token = gettoken(&ptr, value, sizeof(value) - 1);
 					if (token < T_EQSTART || token > T_EQEND) {
-						token = T_OP_EQ;
+						token = (is_check) ? T_OP_CMP_EQ : T_OP_EQ;
 					} else {
-						gettoken(&ptr, value, sizeof(value));
+						gettoken(&ptr, value, sizeof(value) - 1);
 					}
 					if (value[0] == 0) {
 						DEBUG("rlm_ldap: Attribute %s has no value", element->attr);
@@ -1553,7 +1648,10 @@ module_t        rlm_ldap = {
 		ldap_authorize,		/* authorization 	 */
 		NULL,			/* preaccounting 	 */
 		NULL,			/* accounting 		 */
-		NULL			/* checksimul 		 */
+		NULL,			/* checksimul 		 */
+		NULL,			/* pre-proxy 		 */
+		NULL,			/* post-proxy 		 */
+		NULL			/* post-auth 		 */
 	},
 	ldap_detach,		/* detach 		 */
 	NULL,			/* destroy 		 */
