@@ -1,7 +1,7 @@
 /*
  * rlm_mschap.c	
  *
- * Version:	$Id: rlm_mschap.c,v 1.27 2002/06/21 19:57:27 cparker Exp $
+ * Version:	$Id: rlm_mschap.c,v 1.32 2002/10/02 14:37:08 3APA3A Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -693,30 +693,32 @@ static int mschap_authenticate(void * instance, REQUEST *request)
 	struct smb_passwd smbPasswd, smbPasswd1Value, *smbPasswd1 = NULL;
 	AUTHTYPE at = NONE;
 	int res = 0;
+/*
 	int len = 0;
+*/
 	int chap = 0;
 	
-	
-	
 	pdb_init_smb(&smbPasswd);
 	setsmbname(&smbPasswd,request->username->strvalue);
 	password = pairfind(request->config_items, PW_SMB_ACCOUNT_CTRL);
-	if(password){
+	if (password) {
 		smbPasswd.acct_ctrl = password->lvalue;
-		if (smbPasswd.acct_ctrl&ACB_PWNOTREQ) return RLM_MODULE_OK;
+		if (smbPasswd.acct_ctrl & ACB_PWNOTREQ) return RLM_MODULE_OK;
 	}
+
         password = pairfind(request->config_items, PW_SMB_ACCOUNT_CTRL_TEXT);
-        if(password) {
+        if (password) {
 		smbPasswd.acct_ctrl = pdb_decode_acct_ctrl(password->strvalue);
-                if (smbPasswd.acct_ctrl&ACB_PWNOTREQ) return RLM_MODULE_OK;
+                if (smbPasswd.acct_ctrl & ACB_PWNOTREQ) return RLM_MODULE_OK;
         }
+
 	password = pairfind(request->config_items, PW_LM_PASSWORD);
-	if(password){
+	if (password) {
 		if(password->length == 16) {
 			smbPasswd.smb_passwd = password->strvalue;
 			res++;
 		}
-		else if(hex2bin(password->strvalue,smbPasswd.smb_passwd_value,16) != 16) {
+		else if(password->length != 32 || hex2bin(password->strvalue,smbPasswd.smb_passwd_value,16) != 16) {
 			radlog(L_ERR, "rlm_mschap: Invalid LM Password text");
                 } 
                 else {
@@ -725,13 +727,14 @@ static int mschap_authenticate(void * instance, REQUEST *request)
 		}
 		
 	}
+
 	password = pairfind(request->config_items, PW_NT_PASSWORD);
-	if(password){
-		if(password->length == 16){
+	if (password) {
+		if (password->length == 16) {
 			smbPasswd.smb_nt_passwd = password->strvalue;
 			res++;
 		}
-		else if(hex2bin(password->strvalue,smbPasswd.smb_nt_passwd_value,16) != 16) {
+		else if(password->length != 32 || hex2bin(password->strvalue,smbPasswd.smb_nt_passwd_value,16) != 16) {
 			radlog(L_ERR, "rlm_mschap: Invalid NT Password text");
                 }
                 else {
@@ -754,15 +757,30 @@ static int mschap_authenticate(void * instance, REQUEST *request)
 	 */
 
 	password = pairfind(request->packet->vps, PW_PASSWORD);
-	if (password && request->username && *request->username->strvalue!= 0) {
+	if (password && request->username && 
+	    (request->username->strvalue[0] != '\0')) {
 		at = CLEARTEXT;
 		smbPasswd1 = createsmbpw(&smbPasswd1Value,request->username->strvalue, password->strvalue);
-		if ( (smbPasswd.smb_passwd && !memcmp(smbPasswd1->smb_passwd, smbPasswd.smb_passwd, 16)) ||
-			(smbPasswd.smb_nt_passwd && !memcmp(smbPasswd1->smb_nt_passwd, smbPasswd.smb_nt_passwd, 16)) )
-			return RLM_MODULE_OK;
-		else return RLM_MODULE_REJECT;
+		if (smbPasswd.smb_passwd) {
+			if (memcmp(smbPasswd1->smb_passwd, smbPasswd.smb_passwd, 16) == 0) {
+				return RLM_MODULE_OK;
+			}
+			DEBUG2("rlm_mschap: Found SMB password, but user supplied password is incorrect");
+
+		} else if (smbPasswd.smb_nt_passwd) {
+			if (memcmp(smbPasswd1->smb_nt_passwd, smbPasswd.smb_nt_passwd, 16)) {
+				return RLM_MODULE_OK;
+			}
+			DEBUG2("rlm_mschap: Found NT Password, but user supplied password is incorrect");
+		} else {
+			DEBUG2("rlm_mschap: No SMB or NT Password was configured for the user: Rejecting them.");
+		}
+			
+		return RLM_MODULE_REJECT;
 	}
-	else if ( (challenge = pairfind(request->packet->vps, PW_MSCHAP_CHALLENGE)) ){
+
+	challenge = pairfind(request->packet->vps, PW_MSCHAP_CHALLENGE);
+	if (challenge) {
 		/*
 		 *	We need an MS-CHAP-Challenge attribute to calculate
 		 *	the response.
@@ -781,6 +799,7 @@ static int mschap_authenticate(void * instance, REQUEST *request)
 			/*
 			 * Try NT response first if UseNT flag is set
 			 */
+			        DEBUG2("rlm_mschap: doing MS-CHAPv1 with NT-Password");
 				mschap(challenge->strvalue, &smbPasswd, calculated, 1);
 				if (memcmp(response->strvalue + 26, calculated, 24) == 0) {
 					res = RLM_MODULE_OK;
@@ -791,6 +810,7 @@ static int mschap_authenticate(void * instance, REQUEST *request)
 			/*
 			 *	Use LM response.
 			 */
+			        DEBUG2("rlm_mschap: doing MS-CHAPv1 with LM-Password");
 				mschap(challenge->strvalue, &smbPasswd, 
 					calculated, 0);
 				if (memcmp(response->strvalue + 2, calculated, 24) == 0) {
@@ -809,6 +829,7 @@ static int mschap_authenticate(void * instance, REQUEST *request)
 			 *	We need NT hash for it to calculate response
 			 */
 			if (smbPasswd.smb_nt_passwd) {
+			        DEBUG2("rlm_mschap: doing MS-CHAPv2 with NT-Password");
 				mschap2(response->strvalue + 2,  challenge->strvalue,
 					&smbPasswd, calculated);
 				if (memcmp(response->strvalue + 26, calculated, 24) == 0) {
@@ -822,13 +843,15 @@ static int mschap_authenticate(void * instance, REQUEST *request)
 					chap = 2;
 				}
 			}
-		}
-		else {
+
+		} else {
 			radlog(L_AUTH, "rlm_mschap: Response attribute is not found");
 			return RLM_MODULE_INVALID;
 		}
-		if (res == RLM_MODULE_OK){
-			if (smbPasswd.acct_ctrl&ACB_AUTOLOCK) {
+
+		if (res == RLM_MODULE_OK) {
+			if (smbPasswd.acct_ctrl & ACB_AUTOLOCK) {
+				radlog(L_AUTH, "rlm_mschap: Account locked out");
 				add_reply( &request->reply->vps, *response->strvalue,
 					"MS-CHAP-Error", "E=647 R=0", 9);
 				return RLM_MODULE_USERLOCK;
@@ -837,6 +860,7 @@ static int mschap_authenticate(void * instance, REQUEST *request)
 			/* now create MPPE attributes */
 			if (inst->use_mppe) {
 				if (chap == 1){
+				        DEBUG2("rlm_mschap: adding MS-CHAPv1 MPPE keys");
 					memset (mppe_sendkey, 0, 32);
 					if (smbPasswd.smb_passwd) 
 						memcpy(mppe_sendkey, smbPasswd.smb_passwd, 8);
@@ -851,13 +875,15 @@ static int mschap_authenticate(void * instance, REQUEST *request)
 						memcpy (mppe_sendkey+8,smbPasswd.smb_nt_passwd,16);   
 					*/
 						md4_calc(mppe_sendkey+8, smbPasswd.smb_nt_passwd,16);
-					len = 32;
+/*
 					rad_pwencode(mppe_sendkey, &len, 
 						 request->secret, request->packet->vector);
+*/
 					mppe_add_reply( &request->reply->vps,
-						"MS-CHAP-MPPE-Keys",mppe_sendkey,len);
+						"MS-CHAP-MPPE-Keys",mppe_sendkey,32);
 				}
 				else if (chap == 2){
+				        DEBUG2("rlm_mschap: adding MS-CHAPv2 MPPE keys");
 					mppe_chap2_gen_keys128(request->secret,request->packet->vector,
 						smbPasswd.smb_nt_passwd,
 						response->strvalue + 26,
@@ -881,7 +907,7 @@ static int mschap_authenticate(void * instance, REQUEST *request)
 				rad_assert(reply_attr != NULL);
 				pairadd(&request->reply->vps, reply_attr);
 				reply_attr = pairmake("MS-MPPE-Encryption-Types",
-					(inst->require_strong)? "0x00000004":"0x0000006",
+					(inst->require_strong)? "0x00000004":"0x00000006",
 					T_OP_EQ);
 				rad_assert(reply_attr != NULL);
 				pairadd(&request->reply->vps, reply_attr);
@@ -889,10 +915,19 @@ static int mschap_authenticate(void * instance, REQUEST *request)
 			}
 			return res;
 		}
+		else {
+		        DEBUG2("rlm_mschap: Authentication failed");
+		}
+	}
+	else{
+		DEBUG2("No MS-CHAP related attributes in request");
+		return RLM_MODULE_REJECT;
 	}
 	
 	add_reply( &request->reply->vps, *response->strvalue,
 		"MS-CHAP-Error", "E=691 R=1", 9);
+	DEBUG2("rlm_mschap: Nothing in the packet I recognise: Rejecting the user");
+
 	return RLM_MODULE_REJECT;
 #undef inst
 }
@@ -907,7 +942,10 @@ module_t rlm_mschap = {
 	  mschap_authorize,	/* authorize */
 	  NULL,			/* pre-accounting */
 	  NULL,			/* accounting */
-	  NULL			/* checksimul */
+	  NULL,			/* checksimul */
+	  NULL,			/* pre-proxy */
+	  NULL,			/* post-proxy */
+	  NULL			/* post-auth */
   },
   mschap_detach,		/* detach */
   NULL,				/* destroy */
