@@ -3,7 +3,7 @@
  *		Contains the functions for the "huntgroups" and "hints"
  *		files.
  *
- * Version:     $Id: rlm_preprocess.c,v 1.39 2002/07/19 14:34:06 aland Exp $
+ * Version:     $Id: rlm_preprocess.c,v 1.41 2002/08/24 16:54:59 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@
  * Copyright 2000  Alan DeKok <aland@ox.org>
  */
 
-static const char rcsid[] = "$Id: rlm_preprocess.c,v 1.39 2002/07/19 14:34:06 aland Exp $";
+static const char rcsid[] = "$Id: rlm_preprocess.c,v 1.41 2002/08/24 16:54:59 aland Exp $";
 
 #include	"autoconf.h"
 #include	"libradius.h"
@@ -123,7 +123,7 @@ static void cisco_vsa_hack(VALUE_PAIR *vp)
 
 		if (vendorpec != 9) continue; /* not a Cisco VSA, continue */
 
-		if ((vp->attribute & 0xffff) != 1) continue; /* Cisco-AVPair */
+		if ((vp->attribute & 0xffff) == 1) continue; /* Cisco-AVPair */
 
 		/*
 		 *  We strip out the duplicity from the value field,
@@ -756,7 +756,10 @@ module_t rlm_preprocess = {
 		preprocess_authorize,	/* authorization */
 		preprocess_preaccounting, /* pre-accounting */
 		NULL,			/* accounting */
-		NULL			/* checksimul */
+		NULL,			/* checksimul */
+		NULL,			/* pre-proxy */
+		NULL,			/* post-proxy */
+		NULL			/* post-auth */
 	},
 	preprocess_detach,	/* detach */
 	NULL,			/* destroy */
