@@ -1,7 +1,7 @@
 /*
  * rlm_radutmp.c	
  *
- * Version:	$Id: rlm_radutmp.c,v 1.13 2002/05/23 14:44:55 aland Exp $
+ * Version:	$Id: rlm_radutmp.c,v 1.14 2002/08/24 16:55:00 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -634,7 +634,10 @@ module_t rlm_radutmp = {
 	  NULL,                 /* authorization */
 	  NULL,                 /* preaccounting */
 	  radutmp_accounting,   /* accounting */
-	  radutmp_checksimul	/* checksimul */
+	  radutmp_checksimul,	/* checksimul */
+	  NULL,			/* pre-proxy */
+	  NULL,			/* post-proxy */
+	  NULL			/* post-auth */
   },
   radutmp_detach,               /* detach */
   NULL,         	        /* destroy */
