@@ -1,7 +1,7 @@
 /*
  * rlm_realm.c	
  *
- * Version:	$Id: rlm_realm.c,v 1.41 2002/07/18 17:09:18 cparker Exp $
+ * Version:	$Id: rlm_realm.c,v 1.43 2002/08/24 16:55:00 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -39,7 +39,7 @@
 #include "radiusd.h"
 #include "modules.h"
 
-static const char rcsid[] = "$Id: rlm_realm.c,v 1.41 2002/07/18 17:09:18 cparker Exp $";
+static const char rcsid[] = "$Id: rlm_realm.c,v 1.43 2002/08/24 16:55:00 aland Exp $";
 
 #define  REALM_FORMAT_PREFIX   0
 #define  REALM_FORMAT_SUFFIX   1
@@ -139,9 +139,18 @@ static REALM *check_for_realm(void *instance, REQUEST *request)
 		break;
 	}
 
-	DEBUG2("    rlm_realm: Looking up realm %s for User-Name = \"%s\"",
-	       (realmname == NULL) ? "NULL" : realmname,
-	       request->username->strvalue);
+	/*
+	 *	Print out excruciatingly descriptive debugging messages
+	 *	for the people who find it too difficult to think about
+	 *	what's going on.
+	 */
+	if (realmname) {
+		DEBUG2("    rlm_realm: Looking up realm %s for User-Name = \"%s\"",
+		       realmname, request->username->strvalue);
+	} else {
+		DEBUG2("    rlm_realm: No '%c' in User-Name = \"%s\", looking up realm NULL",
+		       inst->delim[0], request->username->strvalue);
+	}
 
 	/*
 	 *	Allow NULL realms.
@@ -411,7 +420,10 @@ module_t rlm_realm = {
 	  realm_authorize,	/* authorization */
 	  realm_preacct,	/* preaccounting */
 	  NULL,			/* accounting */
-	  NULL			/* checksimul */
+	  NULL,			/* checksimul */
+	  NULL,			/* pre-proxy */
+	  NULL,			/* post-proxy */
+	  NULL			/* post-auth */
   },
   realm_detach,			/* detach */
   NULL,				/* destroy */
