@@ -2,7 +2,7 @@
  *  sql.c		rlm_sql - FreeRADIUS SQL Module
  *		Main code directly taken from ICRADIUS
  *
- * Version:	$Id: sql.c,v 1.58 2002/11/08 20:45:10 aland Exp $
+ * Version:	$Id: sql.c,v 1.58.2.2 2002/12/06 18:00:25 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -338,11 +338,16 @@ int rlm_sql_fetch_row(SQLSOCK *sqlsocket, SQL_INST *inst) {
 	ret = (inst->module->sql_fetch_row)(sqlsocket, inst->config);
 
 	if (ret == SQL_DOWN) {
+	        /* close the socket that failed */
+	        (inst->module->sql_close)(sqlsocket, inst->config);
+
+		/* reconnect the socket */
 		if (connect_single_socket(sqlsocket, inst) < 0) {
 			radlog(L_ERR, "rlm_sql (%s): reconnect failed, database down?", inst->config->xlat_name);
 			return -1;
 		}
 
+		/* retry the query on the newly connected socket */
 		ret = (inst->module->sql_fetch_row)(sqlsocket, inst->config);
 
 		if (ret) {
@@ -365,14 +370,26 @@ int rlm_sql_fetch_row(SQLSOCK *sqlsocket, SQL_INST *inst) {
 int rlm_sql_query(SQLSOCK *sqlsocket, SQL_INST *inst, char *query) {
 	int ret;
 
+	/*
+	 *	If there's no query, return an error.
+	 */
+	if (!query || !*query) {
+		return -1;
+	}
+
 	ret = (inst->module->sql_query)(sqlsocket, inst->config, query);
 
 	if (ret == SQL_DOWN) {
+	        /* close the socket that failed */
+	        (inst->module->sql_close)(sqlsocket, inst->config);
+
+		/* reconnect the socket */
 		if (connect_single_socket(sqlsocket, inst) < 0) {
 			radlog(L_ERR, "rlm_sql (%s): reconnect failed, database down?", inst->config->xlat_name);
 			return -1;
 		}
 
+		/* retry the query on the newly connected socket */
 		ret = (inst->module->sql_query)(sqlsocket, inst->config, query);
 
 		if (ret) {
@@ -395,14 +412,26 @@ int rlm_sql_query(SQLSOCK *sqlsocket, SQL_INST *inst, char *query) {
 int rlm_sql_select_query(SQLSOCK *sqlsocket, SQL_INST *inst, char *query) {
 	int ret;
 
+	/*
+	 *	If there's no query, return an error.
+	 */
+	if (!query || !*query) {
+		return -1;
+	}
+
 	ret = (inst->module->sql_select_query)(sqlsocket, inst->config, query);
 
 	if (ret == SQL_DOWN) {
+	        /* close the socket that failed */
+	        (inst->module->sql_close)(sqlsocket, inst->config);
+
+		/* reconnect the socket */
 		if (connect_single_socket(sqlsocket, inst) < 0) {
 			radlog(L_ERR, "rlm_sql (%s): reconnect failed, database down?", inst->config->xlat_name);
 			return -1;
 		}
 
+		/* retry the query on the newly connected socket */
 		ret = (inst->module->sql_select_query)(sqlsocket, inst->config, query);
 
 		if (ret) {
@@ -428,6 +457,13 @@ int sql_getvpdata(SQL_INST * inst, SQLSOCK * sqlsocket, VALUE_PAIR **pair, char
 	SQL_ROW row;
 	int     rows = 0;
 
+	/*
+	 *	If there's no query, return an error.
+	 */
+	if (!query || !*query) {
+		return -1;
+	}
+
 	if (rlm_sql_select_query(sqlsocket, inst, query)) {
 		radlog(L_ERR, "rlm_sql_getvpdata: database query error");
 		return -1;
